/*
 * ataks.c - C source for GNU CHESS
 *
 * Revision: 1991-04-15
 *
 * Copyright (C) 1986, 1987, 1988, 1989, 1990 Free Software Foundation, Inc.
 * Copyright (c) 1988, 1989, 1990  John Stanback
 *
 * This file is part of CHESS.
 *
 * CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY.  No author or distributor accepts responsibility to anyone for
 * the consequences of using it or for whether it serves any particular
 * purpose or works at all, unless he says so in writing.  Refer to the CHESS
 * General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute CHESS, but
 * only under the conditions described in the CHESS General Public License. A
 * copy of this license is supposed to have been given to you along with
 * CHESS so you can know your rights and responsibilities.  It should be in a
 * file named COPYING.  Among other things, the copyright notice and this
 * notice must be preserved on all copies.
 */
#include "schah.h"
void
ataks (short int side, short int *a)

/*
 * Fill array atak[][] with info about ataks to a square.  Bits 8-15 are set
 * if the piece (king..pawn) ataks the square.  Bits 0-7 contain a count of
 * total ataks to the square.
 */

{
  register short u, c, sq;
  register unsigned char *ppos, *pdir;
  short i, piece, *PL;

#ifdef NOMEMSET
  for (u = 64; u; a[--u] = 0) ;
#else
  memset ((char *) a, 0, 64 * sizeof (a[0]));
#endif /* NOMEMSET */
  PL = PieceList[side];
  for (i = PieceCnt[side]; i >= 0; i--)
    {
      sq = PL[i];
      piece = board[sq];
      c = control[piece];
      if (sweep[piece])
	{
	  ppos = nextpos[piece][sq];
	  pdir = nextdir[piece][sq];
	  u = ppos[sq];
	  do
	    {
	      a[u] = (++a[u] | c);
	      u = ((color[u] == neutral) ? ppos[u] : pdir[u]);
	  } while (u != sq);
	}
      else
	{
	  pdir = nextdir[ptype[side][piece]][sq];
	  u = pdir[sq];		/* follow captures thread for pawns */
	  do
	    {
	      a[u] = (++a[u] | c);
	      u = pdir[u];
	  } while (u != sq);
	}
    }
}
