/*
 * genmoves.c - C source for GNU CHESS
 *
 * Revision: 1991-04-15
 *
 * Copyright (C) 1986, 1987, 1988, 1989, 1990 Free Software Foundation, Inc.
 * Copyright (c) 1988, 1989, 1990  John Stanback
 *
 * This file is part of CHESS.
 *
 * CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY.  No author or distributor accepts responsibility to anyone for
 * the consequences of using it or for whether it serves any particular
 * purpose or works at all, unless he says so in writing.  Refer to the CHESS
 * General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute CHESS, but
 * only under the conditions described in the CHESS General Public License. A
 * copy of this license is supposed to have been given to you along with
 * CHESS so you can know your rights and responsibilities.  It should be in a
 * file named COPYING.  Among other things, the copyright notice and this
 * notice must be preserved on all copies.
 */
#include "schah.h"

#define Link(from,to,flag,s) \
{\
   node->f = from; node->t = to;\
     node->reply = 0;\
       node->flags = flag;\
	 node->score = s;\
	   ++node;\
	     ++TrPnt[ply+1];\
	     }

inline void
LinkMove (short int ply,
	  short int f,
	  short int t,
	  short int flag,
	  short int xside)

/*
 * Add a move to the tree.  Assign a bonus to order the moves as follows: 1.
 * Principle variation 2. Capture of last moved piece 3. Other captures
 * (major pieces first) 4. Killer moves 5.
 */

{
  register short s;
  register unsigned short mv;
  register struct leaf *node;

  node = &Tree[TrPnt[ply + 1]];
  mv = (f << 8) | t;
  s = killt[mv|sidebit];
  if (color[t] != neutral)
    {
      /* TOsquare is the square the last piece moved moved to */
      s += (value[board[t]] - board[f] + ((t == TOsquare) ? 500 : 0));
    }
  if (board[f] == pawn)
    if (row (t) == 0 || row (t) == 7)
      {
	flag |= promote;
	s += 800;
#if !defined XBOARD && !defined CHESSTOOL
	Link (f, t, flag | queen, s - 20000);
	s -= 200;
	Link (f, t, flag | knight, s - 20000);
	s -= 50;
	Link (f, t, flag | rook, s - 20000);
	flag |= bishop;
	s -= 50;
#else
	flag |= queen;
#endif
      }
    else if (row (t) == 1 || row (t) == 6)
      {
	flag |= pwnthrt;
	s += 600;
      }
  Link (f, t, flag, s - 20000);
}

inline
void
GenMoves (register short int ply, register short int sq, short int side, short int xside)

/*
 * Generate moves for a piece. The moves are taken from the precalulated
 * array nextpos/nextdir. If the board is free, next move is choosen from
 * nextpos else from nextdir.
 */

{
  register short u, piece;
  register unsigned char *ppos, *pdir;

  piece = board[sq];
  ppos = nextpos[ptype[side][piece]][sq];
  pdir = nextdir[ptype[side][piece]][sq];
  if (piece == pawn)
    {
      u = ppos[sq];		/* follow no captures thread */
      if (color[u] == neutral)
	{
	  LinkMove (ply, sq, u, 0, xside);
	  u = ppos[u];
	  if (color[u] == neutral)
	    LinkMove (ply, sq, u, 0, xside);
	}
      u = pdir[sq];		/* follow captures thread */
      if (color[u] == xside)
	LinkMove (ply, sq, u, capture, xside);
      u = pdir[u];
      if (color[u] == xside)
	LinkMove (ply, sq, u, capture, xside);
    }
  else
    {
      u = ppos[sq];
      do
	{
	  if (color[u] == neutral)
	    {
	      LinkMove (ply, sq, u, 0, xside);
	      u = ppos[u];
	    }
	  else
	    {
	      if (color[u] == xside)
		LinkMove (ply, sq, u, capture, xside);
	      u = pdir[u];
	    }
      } while (u != sq);
    }
}

void
MoveList (short int side, register short int ply)

/*
 * Fill the array Tree[] with all available moves for side to play. Array
 * TrPnt[ply] contains the index into Tree[] of the first move at a ply.
 */

{
  register short i, xside, f;

  xside = otherside[side];
  TrPnt[ply + 1] = TrPnt[ply];
  if (!PV)
    Swag0 = killr0[ply];
  Swag1 = killr1[ply];
  Swag2 = killr2[ply];
  Swag3 = killr3[ply];
  if (ply > 2)
    Swag4 = killr1[ply - 2];
  sidebit = ((side == white) ? 0 : 0x80);
  killt[SwagHt|sidebit]+=5000;
  killt[Swag0|sidebit]+=2000;
  killt[Swag1|sidebit]+=60;
  killt[Swag2|sidebit]+=50;
  killt[Swag3|sidebit]+=40;
  killt[Swag4|sidebit]+=30;
  for (i = PieceCnt[side]; i >= 0; i--)
    GenMoves (ply, PieceList[side][i], side, xside);
  if (!castld[side])
    {
      f = PieceList[side][0];
      if (castle (side, f, f + 2, 0))
	{
	  LinkMove (ply, f, f + 2, cstlmask, xside);
	}
      if (castle (side, f, f - 2, 0))
	{
	  LinkMove (ply, f, f - 2, cstlmask, xside);
	}
    }
  if (epsquare>0) {
    f = epmove1[epsquare];
    if (color[f] == side && board[f] == pawn)
      LinkMove (ply, f, epsquare, capture | epmask, xside);
    f = epmove2[epsquare];
    if (color[f] == side && board[f] == pawn)
      LinkMove (ply, f, epsquare, capture | epmask, xside);
  }
  killt[SwagHt|sidebit]-=5000;
  killt[Swag0|sidebit]-=2000;
  killt[Swag1|sidebit]-=60;
  killt[Swag2|sidebit]-=50;
  killt[Swag3|sidebit]-=40;
  killt[Swag4|sidebit]-=30;
  SwagHt = 0; /* SwagHt is only used once */
}

void
CaptureList (register short int side, short int ply)

/*
 * Fill the array Tree[] with all available cature and promote moves for side
 * to play. Array TrPnt[ply] contains the index into Tree[] of the first move
 * at a ply.
 */

{
  register short u, sq, xside;
  register struct leaf *node;
  register unsigned char *ppos, *pdir;
  short i, piece, *PL, r7;

  xside = otherside[side];
  TrPnt[ply + 1] = TrPnt[ply];
  node = &Tree[TrPnt[ply]];
  r7 = rank7[side];
  PL = PieceList[side];
  sidebit = ((side == white) ? 0 : 0x80);
  killt[SwagHt|sidebit]+=5000;
  killt[Swag0|sidebit]+=2000;
  killt[Swag1|sidebit]+=60;
  killt[Swag2|sidebit]+=50;
  killt[Swag3|sidebit]+=40;
  killt[Swag4|sidebit]+=30;
  for (i = 0; i <= PieceCnt[side]; i++)
    {
      sq = PL[i];
      piece = board[sq];
      if (sweep[piece])
	{
	  ppos = nextpos[piece][sq];
	  pdir = nextdir[piece][sq];
	  u = ppos[sq];
	  do
	    {
	      if (color[u] == neutral) u = ppos[u];
	      else
		{
		  if (color[u] == xside)
		    Link (sq, u, capture, value[board[u]] + svalue[board[u]] - piece);
		  u = pdir[u];
		}
	  } while (u != sq);
	}
      else
	{
	  pdir = nextdir[ptype[side][piece]][sq];
	  if (piece == pawn && row (sq) == r7)
	    {
	      u = pdir[sq];
	      if (color[u] == xside) Link (sq, u, capture | promote | queen, valueQ);
	      u = pdir[u];
	      if (color[u] == xside) Link (sq, u, capture | promote | queen, valueQ);
	      ppos = nextpos[ptype[side][piece]][sq];
	      u = ppos[sq];	/* also generate non capture promote */
	      if (color[u] == neutral) Link (sq, u, promote | queen, valueQ);
	    }
	  else
	    {
	      u = pdir[sq];
	      do
		{
		  if (color[u] == xside) Link (sq, u, capture, value[board[u]] + svalue[board[u]] - piece);
		  u = pdir[u];
	      } while (u != sq);
	    }
	}
    }
  killt[SwagHt|sidebit]-=5000;
  killt[Swag0|sidebit]-=2000;
  killt[Swag1|sidebit]-=60;
  killt[Swag2|sidebit]-=50;
  killt[Swag3|sidebit]-=40;
  killt[Swag4|sidebit]-=30;
  SwagHt = 0; /* SwagHt is only used once */
}


